
class ValidationUtils {
  // OrderIDs
  static final RegExp orderIDRegex = RegExp(r'^(?:DG)?[1-9]\d{5}$|^(?:CD)?[1-9]\d{6}$', caseSensitive: false);
  static final RegExp dgOrderIDRegex = RegExp(r'^(?:DG)?[1-9]\d{5}$', caseSensitive: false);
  static final RegExp cdOrderIDRegex = RegExp(r'^(?:CD)?[1-9]\d{6}$', caseSensitive: false);

  // DG Claims
  static final RegExp dgSkylineClaimRegex = RegExp(r'^\d{5,}$');
  static final RegExp dgManualClaimRegex = RegExp(r'^(?:0[1-9]|[1-4][0-9]|5[0-3])[a-zA-Z]{4}$');
  static final RegExp dgWarrantyClaimRegex = RegExp(r'^(?:(?:(?:0[1-9]|[1-4][0-9]|5[0-3])[a-zA-Z]{4})|(?:\d{5,}))\(W\)$');

  // Courier Barcodes
  static final RegExp yodelTrackingRegex = RegExp(r'^J?JD000[0-9]{13}$', caseSensitive: false);
  static final RegExp cPlusTrackingRegex = RegExp(r'^((8T58[0-9]{8}[A-Z0-9]001)|([A-Z0-9]{7}))$', caseSensitive: false);
  static final RegExp dpdTrackingRegex = RegExp(r'^(?:1550)?\d{10}$', caseSensitive: false);
  static final RegExp dpdBarcodeRegex = RegExp(r'^%[0-9a-zA-Z]{4}\d[a-zA-Z0-9]{2}(1550\d{10})\d{6}$', caseSensitive: false);

  // DPD Service Areas
  static final RegExp postcodeRegex = RegExp(r"^([Gg][Ii][Rr] 0[Aa]{2})|((([A-Za-z][0-9]{1,2})|(([A-Za-z][A-Ha-hJ-Yj-y][0-9]{1,2})|(([A-Za-z][0-9][A-Za-z])|([A-Za-z][A-Ha-hJ-Yj-y][0-9]?[A-Za-z]))))\s?[0-9][A-Za-z]{2})$", caseSensitive: false);
  static final RegExp scottishHighlandsOrIslandsPostcodeOutcodeRegex = RegExp(r"^((AB(?:3[1-9]|4\d|5[0-6]))|(FK(?:1[7-9]|2[0-1]))|(HS[1-9])|(IV[1-9]\d?)|(KA2[78])|(KW(?:\d|1[0-7]))|(PA(?:[2-6]\d|7[0-8]))|(PH(?:1[5-9]|[2-4]\d|50))|(ZE[1-3]))$", caseSensitive: false);
  static final RegExp niPostcodeOutcodeRegex = RegExp(r"^(BT[1-9]\d?)$", caseSensitive: false);
  static final RegExp outOfAreaPostcodeOutcodeRegex = RegExp(r"^((IM(?:8[6-7]|99|[1-9]))|(GY(?:10|[1-9]))|(JE[1-5]))$", caseSensitive: false);

  // Phone Numbers
  static final RegExp phoneNumberRegex = RegExp(r'^((\+?44|0)\d{9,10})$');
  static final RegExp mobileNumberRegex = RegExp(r'^((\+?44|0)7\d{9})$');

  // Email
  static final RegExp emailRegex = RegExp(r'^[a-zA-Z0-9._%-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$', caseSensitive: false);

  // Misc
  static final RegExp ipv4Regex = RegExp(r'^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$');



  // DPD Service Areas
  static bool isValidPostcode(String postcode) => postcodeRegex.hasMatch(postcode.trim());
  static bool isScottishHighlandsOrIslandsPostcode(String postcode) => isValidPostcode(postcode) && scottishHighlandsOrIslandsPostcodeOutcodeRegex.hasMatch(_extractOutcodeFromPostcode(postcode));
  static bool isNIPostcode(String postcode) => isValidPostcode(postcode) && niPostcodeOutcodeRegex.hasMatch(_extractOutcodeFromPostcode(postcode));
  static bool isOutOfAreaPostcode(String postcode) => isValidPostcode(postcode) && outOfAreaPostcodeOutcodeRegex.hasMatch(_extractOutcodeFromPostcode(postcode));

  static bool isValidPhoneNumber(String phone) => phoneNumberRegex.hasMatch(_cleanPhoneNumber(phone));
  static bool isValidMobileNumber(String phone) => mobileNumberRegex.hasMatch(_cleanPhoneNumber(phone));

  static String cleanPhoneNumber(String phone) {
    phone = _cleanPhoneNumber(phone);
    if (!isValidPhoneNumber(phone)) throw ArgumentError("Invalid phone number provided");
    return phone.replaceAll(RegExp(r'^(\+?44)', caseSensitive: false), '0');
  }

  static bool isValidEmail(String email) => emailRegex.hasMatch(email.trim());

  static bool isValidIP(String ip) => ipv4Regex.hasMatch(ip.trim());

  // Helpers
  static String _cleanPhoneNumber(String phone) => phone.trim().replaceAll(' ', '');
  static String _extractOutcodeFromPostcode(String postcode) => postcode.trim().split(' ').first;
}
